; EngineControl.s
; Runs on TM4C1294
; Use a pointer implementation of a Mealy finite state machine to operate
; an engine with a control input, gas and brake outputs, and two states.
; Daniel Valvano
; May 8, 2014

;  This example accompanies the book
;  "Embedded Systems: Introduction to Arm Cortex M Microcontrollers",
;  ISBN: 978-1469998749, Jonathan Valvano, copyright (c) 2014
;  Program 6.9
;
;Copyright 2014 by Jonathan W. Valvano, valvano@mail.utexas.edu
;   You may use, edit, run or distribute this file
;   as long as the above copyright notice remains
;THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
;OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
;MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
;VALVANO SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
;OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
;For more information about my classes, my research, and my books, see
;http://users.ece.utexas.edu/~valvano/

; control input connected to PJ0 (negative logic SW1; 0=go)
; gas output connected to PN0 (positive logic LED2)
; brake output connected to PN1 (positive logic LED1)

INPUT                         EQU 0x40060004   ; port J bit 0
GPIO_PORTJ_DIR_R              EQU 0x40060400
GPIO_PORTJ_AFSEL_R            EQU 0x40060420
GPIO_PORTJ_PUR_R              EQU 0x40060510
GPIO_PORTJ_DEN_R              EQU 0x4006051C
GPIO_PORTJ_AMSEL_R            EQU 0x40060528
GPIO_PORTJ_PCTL_R             EQU 0x4006052C
OUTPUT                        EQU 0x4006400C   ; port N bits 1-0
GPIO_PORTN_DIR_R              EQU 0x40064400
GPIO_PORTN_AFSEL_R            EQU 0x40064420
GPIO_PORTN_DEN_R              EQU 0x4006451C
GPIO_PORTN_AMSEL_R            EQU 0x40064528
GPIO_PORTN_PCTL_R             EQU 0x4006452C
SYSCTL_RCGCGPIO_R             EQU 0x400FE608
SYSCTL_RCGCGPIO_R12           EQU 0x00001000  ; GPIO Port N Run Mode Clock Gating Control
SYSCTL_RCGCGPIO_R8            EQU 0x00000100  ; GPIO Port J Run Mode Clock Gating Control
SYSCTL_PRGPIO_R               EQU 0x400FEA08
SYSCTL_PRGPIO_R12             EQU 0x00001000  ; GPIO Port N Peripheral Ready
SYSCTL_PRGPIO_R8              EQU 0x00000100  ; GPIO Port J Peripheral Ready

        AREA    |.text|, CODE, READONLY, ALIGN=2
        THUMB
        EXPORT   Start
        IMPORT   PLL_Init
        IMPORT   SysTick_Init
        IMPORT   SysTick_Wait10ms
;Linked data structure
;Put in ROM
OUT   EQU 0    ;offset for output
DELAY EQU 8    ;offset for delay between states
NEXT  EQU 12   ;offset for next
Stop  DCD 2,0     ;Outputs for 0,1
      DCD 10      ;100 ms
      DCD Stop,Go ;Next for 0,1
Go    DCD 0,1     ;Outputs for 0,1
      DCD 10      ;100 ms
      DCD Stop,Go ;Next for 0,1

Start
    BL  PLL_Init      ; 120 MHz clock
    BL  SysTick_Init  ; enable SysTick
    ; activate clock for Port J
    LDR R1, =SYSCTL_RCGCGPIO_R      ; R1 = SYSCTL_RCGCGPIO_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    ORR R0, R0, #SYSCTL_RCGCGPIO_R8 ; R0 = R0|SYSCTL_RCGCGPIO_R8
    STR R0, [R1]                    ; [R1] = R0
    ; allow time for clock to stabilize
    LDR R1, =SYSCTL_PRGPIO_R        ; R1 = SYSCTL_PRGPIO_R (pointer)
GPIOJinitloop
    LDR R0, [R1]                    ; R0 = [R1] (value)
    ANDS R0, R0, #SYSCTL_PRGPIO_R8  ; R0 = R0&SYSCTL_PRGPIO_R8
    BEQ GPIOJinitloop               ; if(R0 == 0), keep polling
    ; set direction register
    LDR R1, =GPIO_PORTJ_DIR_R       ; R1 = GPIO_PORTJ_DIR_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    BIC R0, R0, #0x01               ; R0 = R0&~0x01 (make PJ0 in (PJ0 built-in SW1))
    STR R0, [R1]                    ; [R1] = R0
    ; set alternate function register
    LDR R1, =GPIO_PORTJ_AFSEL_R     ; R1 = GPIO_PORTJ_AFSEL_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    BIC R0, R0, #0x01               ; R0 = R0&~0x01 (disable alt funct on PJ0)
    STR R0, [R1]                    ; [R1] = R0
    ; set pull-up register
    LDR R1, =GPIO_PORTJ_PUR_R       ; R1 = GPIO_PORTJ_PUR_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    ORR R0, R0, #0x01               ; R0 = R0|0x01 (enable pull-up on PJ0)
    STR R0, [R1]                    ; [R1] = R0
    ; set digital enable register
    LDR R1, =GPIO_PORTJ_DEN_R       ; R1 = GPIO_PORTJ_DEN_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    ORR R0, R0, #0x01               ; R0 = R0|0x01 (enable digital I/O on PJ0)
    STR R0, [R1]                    ; [R1] = R0
    ; set port control register
    LDR R1, =GPIO_PORTJ_PCTL_R      ; R1 = GPIO_PORTJ_PCTL_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    BIC R0, R0, #0x0000000F         ; R0 = R0&0xFFFFFFF0 (clear bit0 field)
    ADD R0, R0, #0x00000000         ; R0 = R0+0x00000000 (configure PJ0 as GPIO)
    STR R0, [R1]                    ; [R1] = R0
    ; set analog mode select register
    LDR R1, =GPIO_PORTJ_AMSEL_R     ; R1 = GPIO_PORTJ_AMSEL_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    BIC R0, R0, #0x01               ; R0 = R0&~0x01 (disable analog functionality on PJ0)
    STR R0, [R1]                    ; [R1] = R0
    ; activate clock for Port N
    LDR R1, =SYSCTL_RCGCGPIO_R      ; R1 = SYSCTL_RCGCGPIO_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    ORR R0, R0, #SYSCTL_RCGCGPIO_R12; R0 = R0|SYSCTL_RCGCGPIO_R12
    STR R0, [R1]                    ; [R1] = R0
    ; allow time for clock to stabilize
    LDR R1, =SYSCTL_PRGPIO_R        ; R1 = SYSCTL_PRGPIO_R (pointer)
GPIONinitloop
    LDR R0, [R1]                    ; R0 = [R1] (value)
    ANDS R0, R0, #SYSCTL_PRGPIO_R12 ; R0 = R0&SYSCTL_PRGPIO_R12
    BEQ GPIONinitloop               ; if(R0 == 0), keep polling
    ; set direction register
    LDR R1, =GPIO_PORTN_DIR_R       ; R1 = GPIO_PORTN_DIR_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    ORR R0, R0, #0x03               ; R0 = R0|0x03 (make PN1-0 out (PN1-0 built-in LED1-2))
    STR R0, [R1]                    ; [R1] = R0
    ; set alternate function register
    LDR R1, =GPIO_PORTN_AFSEL_R     ; R1 = GPIO_PORTN_AFSEL_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    BIC R0, R0, #0x03               ; R0 = R0&~0x03 (disable alt funct on PN1-0)
    STR R0, [R1]                    ; [R1] = R0
    ; set digital enable register
    LDR R1, =GPIO_PORTN_DEN_R       ; R1 = GPIO_PORTN_DEN_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    ORR R0, R0, #0x03               ; R0 = R0|0x03 (enable digital I/O on PN1-0)
    STR R0, [R1]                    ; [R1] = R0
    ; set port control register
    LDR R1, =GPIO_PORTN_PCTL_R      ; R1 = GPIO_PORTN_PCTL_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    BIC R0, R0, #0x000000FF         ; R0 = R0&0xFFFFFF00 (clear bit1-0 field)
    ADD R0, R0, #0x00000000         ; R0 = R0+0x00000000 (configure PN1-0 as GPIO)
    STR R0, [R1]                    ; [R1] = R0
    ; set analog mode select register
    LDR R1, =GPIO_PORTN_AMSEL_R     ; R1 = GPIO_PORTN_AMSEL_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    BIC R0, R0, #0x03               ; R0 = R0&~0x03 (disable analog functionality on PN1-0)
    STR R0, [R1]                    ; [R1] = R0

    LDR R4, =Stop     ; State pointer
    LDR R5, =INPUT    ; 0x40060004
    LDR R6, =OUTPUT   ; 0x4006400C
FSM LDR R3, [R5]      ; read Input
    MVN R3, R3        ; invert Input for negative logic
    AND R3, R3, #0x1  ; only interested in bit 0
    LSL R3, R3, #2    ; 4 bytes each
    ADD R1, R3, #OUT  ; R1 is 0 or 4
    LDR R2, [R4, R1]  ; Output value
    STR R2, [R6]      ; set outputs
    LDR R0, [R4, #DELAY] ; num 10ms delays
    ADD R1, R3, #NEXT ; R1 is 12 or 16
    LDR R4, [R4, R1]  ; find next state
    BL  SysTick_Wait10ms
    B   FSM           ; go to next state

    ALIGN                           ; make sure the end of this section is aligned
    END                             ; end of file
